/*
Author: Adrian Kuegel
Date: August 23rd, 2011
Algorithm: Binary Indexed Tree (aka Fenwick Tree)
Complexity: O(n log (n + l))
*/

#include <stdio.h>
#include <string.h>
#include <assert.h>

#define MAXL 100000
#define MAXN 100000

int size, bi_tree[MAXL+MAXN], pos[MAXL];

void add(int pos, int val) {
	if (!pos) {
		bi_tree[0] += val;
		return;
	}
	while(pos < size) {
		bi_tree[pos] += val;
		pos += pos&(-pos);
	}
}

int rank(int pos) {
	int res = bi_tree[0];
	while(pos) {
		res += bi_tree[pos];
		pos &= pos-1;
	}
	return res;
}

int main() {
	int tc, n, l, a;
	scanf("%d", &tc);
	while(tc--) {
		scanf("%d %d", &l, &n);
		assert(n >= 1 && n <= MAXN);
		assert(l >= 1 && l <= MAXN);
		size = l + n;
		memset(bi_tree, 0, sizeof(int) * (l + n));
		for (int i=0; i<l; ++i) {
			pos[i] = n + i;
			add(n + i, 1);
		}
		for (int i=n-1; i>=0; --i) {
			scanf("%d", &a);
			assert(a >= 1 && a <= l);
			--a;
			printf("%d%c", rank(pos[a])-1, i?' ':'\n');
			add(pos[a], -1);
			add(pos[a] = i, 1);
		}
	}
	return 0;
}
