import java.io.*;
import java.util.*;

public class jeroen {
    public static int[] dx = {0, -1, 0, 1};
    public static int[] dy = {1, 0, -1, 0};
    private static String[] grid;

    public static void main(String[] args) {
	Scanner sc = new Scanner(System.in);
	int tests = sc.nextInt();
	while(tests-- > 0) System.out.println(Testcase(sc) ? "YES" : "NO");
    }

    public static boolean Testcase (Scanner sc) {
	int n = sc.nextInt();
	int m = sc.nextInt();
	grid = new String[n];
	for(int i = 0; i < n; i++)
	    grid[i] = sc.next();
	
	// First count all black and white squares and assign a unique
	// number [1,5,9..] to each black square
	int[][] bnum = new int[n][m];
	int nb = 0;
	int nw = 0;
	for(int i = 0; i < n; i++) {
	    for(int j = 0; j < m; j++) {
		if(get(i,j) == 'B') bnum[i][j] = 1 + 4 * nb++;
		if(get(i,j) == 'W') nw++;
	    }
	}
	
	// We must have twice as many black as white squares
	if(nb * 2 != nw) return false;
	
	TwoSAT tws = new TwoSAT(nb * 4);
	for(int i = 0; i < n; i++) {
	    for(int j = 0; j < m; j++) {
		if(get(i,j) == 'B') {
		    // A black square must match to exactly one
		    // white square from {left,right} and {up,down}
		    for(int d = 0; d < 2; d++) {
			char adj1 = get(i+dx[d],j+dy[d]);
			char adj2 = get(i+dx[d+2],j+dy[d+2]);
			if(adj1 == 'W' && adj2 == 'W') {
			    tws.AddClause(bnum[i][j]+d,bnum[i][j]+2+d);
			    tws.AddClause(-(bnum[i][j]+d),-(bnum[i][j]+2+d));
			} else if(adj1 == 'W') {
			    tws.AddClause(bnum[i][j]+d,bnum[i][j]+d);
			} else if(adj2 == 'W') {
			    tws.AddClause(bnum[i][j]+2+d,bnum[i][j]+2+d);
			} else return false;
		    }
		} else if(get(i,j) == 'W') {
		    // A white square can match at most 1 black square
		    for(int d1 = 0; d1 < 4; d1++) {
			for(int d2 = d1 + 1; d2 < 4; d2++) {
			    if(get(i+dx[d1],j+dy[d1]) == 'B' && get(i+dx[d2],j+dy[d2]) == 'B') {
				tws.AddClause(-(bnum[i+dx[d1]][j+dy[d1]]+(d1+2)%4),
					      -(bnum[i+dx[d2]][j+dy[d2]]+(d2+2)%4));
			    }
			}
		    }
		}
	    }
	}

	return tws.Satisfiable();
    }

    // get a square from the grid, return '.' if out of bounds
    private static char get(int i, int j) {
	if(i < 0 || i >= grid.length || j < 0 || j >= grid[0].length()) return '.';
	return grid[i].charAt(j);
    }

    private static class TwoSAT {
	private int n;
	private LinkedList<Integer>[] Neighbours;
	private LinkedList<Integer>[] NeighboursR;

	// create 2-sat solver for n variables
	// turn stupid java warning off
	@SuppressWarnings("unchecked")
	public TwoSAT(int n) {
	    this.n = n;
	    Neighbours = new LinkedList[2 * n];
	    NeighboursR = new LinkedList[2 * n];
	    for (int i = 0; i < 2 * n; i++) {
		Neighbours[i] = new LinkedList<Integer>();
		NeighboursR[i] = new LinkedList<Integer>();
	    }
	}

	// use index 1..n for true and -1..-n for false
	// indicating that v1 or v2 (or both) must be true
	public void AddClause(int v1, int v2) {
	    AddEdge(getNum(-v1), getNum(v2));
	    AddEdge(getNum(-v2), getNum(v1));
	}

	private void AddEdge(int n1, int n2) {
	    Neighbours[n1].add(n2);
	    NeighboursR[n2].add(n1);
	}

	public boolean Satisfiable() {
	    scc();
	    for(int i = 1; i <= n; i++)
		if(component[getNum(i)] == component[getNum(-i)])
		    return false;
	    return true;
	}

	private int getNum(int num) {
	    return num < 0 ? n - num - 1 : num - 1;
	}

	// Strongly Connected Component part
	private int[] component, order;
	private int order_num, actnum;

	private void dfs1(int i) {
	    if(component[i] != 0) return;
	    component[i] = -1;
	    for(int adj : Neighbours[i])
		dfs1(adj);
	    order[order_num++] = i;
	}

	private void dfs2(int i) {
	    if(component[i] != -1) return;
	    component[i] = actnum;
	    for(int adj : NeighboursR[i])
		dfs2(adj);
	}

	private void scc() {
	    component = new int[n * 2];
	    order = new int[n * 2];
	    order_num = 0;

	    for(int i = 0; i < n * 2; i++)
		dfs1(i);
	    
	    actnum = 0;
	    for(int i = n * 2 - 1; i >= 0; i--) {
		if(component[order[i]] == -1) {
		    dfs2(order[i]);
		    actnum++;
		}
	    }
	}
    }
}