// Author: Adrian Kuegel
// Date: August 24th, 2011
// Algorithm: 2SAT
// Complexity: O(n * m)

#include <stdio.h>
#include <vector>
using namespace std;

typedef vector<int> VI;
typedef vector<VI> VVI;

void dfs(int cur, VVI &a, VI &visit, VI &res) {
	if (visit[cur])
		return;
	visit[cur] = 1;
	for (VI::iterator it=a[cur].begin(); it!=a[cur].end(); ++it)
		dfs(*it, a, visit, res);
	res.push_back(cur);
}

VVI SCC(VVI &a) {
	int n = a.size();
	VI toporder, temp, visit(n, 0);
	VVI res, a_rev(n);
	for (int i=0; i<n; ++i) {
		dfs(i, a, visit, toporder);
		for (VI::iterator it=a[i].begin(); it!=a[i].end(); ++it)
			a_rev[*it].push_back(i);
	}
	visit.assign(n, 0);
	for (VI::reverse_iterator it=toporder.rbegin(); it!=toporder.rend(); ++it) {
		if (visit[*it])
			continue;
		temp.clear();
		dfs(*it, a_rev, visit, temp);
		res.push_back(temp);
	}
	return res;
}

bool SAT2(int n, VVI &a, VI &sol)
{
    VVI scc = SCC(a);
    VI sccidx(n<<1);
    for (int i=0; i<(int)scc.size(); i++)
        for (int j=0; j<(int)scc[i].size(); j++)
            sccidx[scc[i][j]] = i;
    sol.resize(n);
    for (int i=0; i<n; sol[i] = sccidx[i]>sccidx[i+n],i++)
        if (sccidx[i] == sccidx[i+n])
            return false;
    return true;
}

#define pb push_back
#define neg(x) (((x)+N)%NN)
#define do_or(x,y) a[neg(x)].pb(y),a[neg(y)].pb(x)
#define do_true(x) a[neg(x)].pb(x)
#define do_xor(x,y) do_or(x,y),do_or(neg(x),neg(y))

char grid[1000][1001];
int dx[4] = {1,0,-1,0};
int dy[4] = {0,1,0,-1};
int n, m;

int calc_var(int x, int y, int d) {
	if (d == 0)
		return n * (m-1) + y * (n-1) + x - 1;
	if (d == 2)
		return n * (m-1) + y * (n-1) + x;
	if (d == 1)
		return x * (m-1) + y - 1;
	return x * (m-1) + y;
}

int flood_fill(int x, int y) {
	if (x < 0 || y < 0 || x >= n || y >= m || grid[x][y] == '.')
		return 0;
	int res = (grid[x][y] == 'B')? 2 : -1;
	grid[x][y] = '.';
	for (int i=0; i<4; ++i)
		res += flood_fill(x + dx[i], y + dy[i]);
	return res;
}

int main() {
	int tc;
	scanf("%d", &tc);
	while(tc--) {
		scanf("%d %d", &n, &m);
		for (int i=0; i<n; ++i)
			scanf("%s", grid[i]);
		int N = (m-1) * n + (n-1) * m;
		int NN = N << 1;
		VVI a(NN);
		VI sol;
		// add B constraints
		for (int i=0; i<n; ++i) {
			for (int j=0; j<m; ++j) {
				if (grid[i][j] != 'B')
					continue;
				if (!j || grid[i][j-1] != 'W') {
					if (j+1 == m || grid[i][j+1] != 'W')
						goto imposs;
					do_true(i*(m-1)+j);
				}
				else if (j+1 == m || grid[i][j+1] != 'W')
					do_true(i*(m-1)+j-1);
				else
					do_xor(i*(m-1)+j-1, i*(m-1)+j);
				if (!i || grid[i-1][j] != 'W') {
					if (i+1 == n || grid[i+1][j] != 'W')
						goto imposs;
					do_true(n*(m-1)+j*(n-1)+i);
				}
				else if (i+1 == n || grid[i+1][j] != 'W')
					do_true(n*(m-1)+j*(n-1)+i-1);
				else
					do_xor(n*(m-1)+j*(n-1)+i-1, n*(m-1)+j*(n-1)+i);
			}
		}
		// add W constraints
		for (int i=0; i<n; ++i) {
			for (int j=0; j<m; ++j) {
				if (grid[i][j] != 'W')
					continue;
				for (int d1=0; d1<4; ++d1) {
					int x1 = i + dx[d1];
					int y1 = j + dy[d1];
					if (x1 < 0 || x1 >= n || y1 < 0 || y1 >= m || grid[x1][y1] != 'B')
						continue;
					int var1 = calc_var(x1, y1, d1);
					for (int d2=d1+1; d2<4; ++d2) {
						int x2 = i + dx[d2];
						int y2 = j + dy[d2];
						if (x2 < 0 || x2 >= n || y2 < 0 || y2 >= m || grid[x2][y2] != 'B')
							continue;
						do_or(N + var1, N + calc_var(x2, y2, d2));
					}
				}
			}
		}
		// do a flood-fill to check for 2 * W = B in connected areas
		for (int i=0; i<n; ++i)
			for (int j=0; j<m; ++j)
				if (flood_fill(i, j))
					goto imposs;
		if (SAT2(N, a, sol)) {
			puts("YES");
			continue;
		}
imposs:
		puts("NO");
	}
	return 0;
}
