/*
 *  [NWERC'11] Pool Construction
 *  by: Jan Kuipers
 */

#include <iostream>
#include <queue>
#include <string>
#include <climits>

using namespace std;

const int dy[4] = {-1,0,0,1};
const int dx[4] = {0,-1,1,0};

int X,Y,D,F,B;
vector<string> map;
vector<vector<int> > sourflow, sinkflow;
vector<vector<vector<int> > > flow;

int main () {

  int runs;
  cin >> runs;

  while (runs--) {

    cin >> X >> Y >> D >> F >> B;

    map = vector<string>(Y);
    for (int y=0; y<Y; y++)
      cin >> map[y];

    sourflow = sinkflow = vector<vector<int> >(Y, vector<int>(X, 0));
    flow = vector<vector<vector<int> > >(Y, vector<vector<int> >(X, vector<int>(4, 0)));

    int res=0;
    
    for (int y=0; y<Y; y++) 
      for (int x=0; x<X; x++) {
	bool boundary = y==0 || y==Y-1 || x==0 || x==X-1;
	if (boundary && map[y][x]=='.') {
	  res += F;
	  map[y][x] = '#';
	}
      }

    bool change;
    
    do {
      change = false;
      
      vector<vector<int> > push(Y, vector<int>(X, -1));
      vector<vector<int> > prev(Y, vector<int>(X, -1));
      
      queue<int> q;
      
      for (int y=0; y<Y; y++) 
	for (int x=0; x<X; x++) 
	  if (map[y][x] == '#' ) {
	    bool boundary = y==0 || y==Y-1 || x==0 || x==X-1;
	    int p = boundary ? INT_MAX : D-sourflow[y][x];
	    if (p > 0) {
	      q.push(y);
	      q.push(x);
	      push[y][x] = p;
	    }
	  }

      while (!q.empty()) {
	int y = q.front(); q.pop();
	int x = q.front(); q.pop();
	if (map[y][x]=='.' && sinkflow[y][x]<F) {
	  int p = min(push[y][x],F-sinkflow[y][x]);
	  sinkflow[y][x] += p;
	  while (prev[y][x] != -1) {
	    int d = prev[y][x];
	    flow[y][x][3-d] -= p;
	    y -= dy[d];
	    x -= dx[d];
	    flow[y][x][d] += p;
	  }
	  sourflow[y][x] += p;
	  res += p;
	  change = true;
	  break;
	}

	for (int d=0; d<4; d++) {
	  if (flow[y][x][d]==B) continue;
	  int ny = y+dy[d];
	  int nx = x+dx[d];
	  if (ny<0 || ny>=Y || nx<0 || nx>=X) continue;
	  if (push[ny][nx] != -1) continue;
	  push[ny][nx] = min(push[y][x],B-flow[y][x][d]);
	  prev[ny][nx] = d;
	  q.push(ny);
	  q.push(nx);
	}
      }
    }
    while (change);
    
    cout << res << endl;
  }
  
  return 0;
}
