#include <iostream>
#include <algorithm>
#include <cstring>
#include <cassert>
#include <vector>

using namespace std;

#define MAXWIDTH 50
#define MAXHEIGHT 50
#define NODES ((MAXHEIGHT)*(MAXWIDTH)+2)
#define INFTY 1000000000
#define NODE(x, y, w) (2 + (y)*(w) + (x))

char buildsite[MAXHEIGHT][MAXWIDTH];

vector<vector<int> > node_ids;
vector<vector<int> > cap;
vector<vector<int> > flow;
bool srcside[NODES];
const int SOURCE = 0;
const int SINK = 1;

struct qnode {
    int node, parent, mincap;
} q[NODES];

int low, high;

int get_node_idx(int from, int to)
{
    for (size_t ii = 0; ii < node_ids[from].size(); ++ii)
        if (node_ids[from][ii] == to)
            return ii;
    return -1;
}

void add_edge(int from, int to, int cap1, int cap2)
{
    assert(get_node_idx(from, to) == -1);

    node_ids[from].push_back(to);
    cap[from].push_back(cap1);
    flow[from].push_back(0);

    node_ids[to].push_back(from);
    cap[to].push_back(cap2);
    flow[to].push_back(0);
}

int initbfs(int source, int target) {
    memset(srcside, 0, sizeof srcside);
    srcside[source] = true;

    q[0].node = source;
    q[0].parent = -1;
    q[0].mincap = INFTY;

    low = 0;
    high = 1;

    return target;
}

bool bfs(int target) {
    int from, to;

    while (low < high && (from = q[low].node) != target) {
        for (to = 0; to < cap[from].size(); ++to) {
            if (!srcside[node_ids[from][to]] && cap[from][to] > flow[from][to]) {
                srcside[node_ids[from][to]] = true;

                q[high].node = node_ids[from][to];
                q[high].parent = low;
                q[high].mincap = min(q[low].mincap, cap[from][to] - flow[from][to]);

                ++high;
            }
        }

        ++low;
    }

    return low < high;
}

int maxflow(int source, int target) {
    int i, j, res = 0;

    while (bfs(initbfs(source, target))) {
        res += q[low].mincap;
        for (i = low; i > 0; i = j) {
            j = q[i].parent;
            flow[q[j].node][get_node_idx(q[j].node, q[i].node)] += q[low].mincap;
            flow[q[i].node][get_node_idx(q[i].node, q[j].node)] -= q[low].mincap;
        }
    }

    return res;
}

void dump_sink(int w, int h)
{
    cout << "SINK:" << endl;
    for (int ii = 0; ii < h; ++ii) {
        for (int jj = 0; jj < w; ++jj) {
            cout << cap[NODE(jj, ii, w)][get_node_idx(NODE(jj, ii, w), SINK)] << " ";
        }
        cout << endl;
    }
}

void dump_source(int w, int h)
{
    cout << "SOURCE:" << endl;
    for (int ii = 0; ii < h; ++ii) {
        for (int jj = 0; jj < w; ++jj) {
            cout << cap[SOURCE][get_node_idx(SOURCE, NODE(jj, ii, w))] << " ";
        }
        cout << endl;
    }
}

void dump_solution(int w, int h)
{
    cout << "SOLUTION" << endl;
    for (int ii = 0; ii < h; ++ii) {
        for (int jj = 0; jj < w; ++jj) {
            if (!srcside[NODE(jj, ii, w)])
                cout << '#';
            else
                cout << '.';
        }
        cout << endl;
    }
}

bool is_trivial_solution(int w, int h)
{
    bool first = srcside[NODE(0, 0, w)];

    for (int ii = 0; ii < h; ++ii) {
        for (int jj = 0; jj < w; ++jj) {
            if (first != srcside[NODE(jj, ii, w)])
                return false;
        }
    }
    return true;
}

int main(int argc, char* argv[])
{
    int cases, w, h, d, f, b;
    char c;

    cin >> cases;
    assert (cases <= 100);

    while (cases--) {
        cin >> w >> h;
        cin >> d >> f >> b;

        assert (2 <= w && w <= MAXWIDTH);
        assert (2 <= h && h <= MAXHEIGHT);
        assert (1 <= d && d <= 10000);
        assert (1 <= f && f <= 10000);
        assert (1 <= b && b <= 10000);

        for (int ii = 0; ii < h; ++ii) {
            for (int jj = 0; jj < w; ++jj) {
                cin >> c;
                assert (c == '.' || c == '#');
                buildsite[ii][jj] = c;
            }
        }

        // reset graph
        node_ids = vector<vector<int> >(2 + w*h, vector<int>());
        cap = vector<vector<int> >(2 + w*h, vector<int>());
        flow = vector<vector<int> >(2 + w*h, vector<int>());

        int cost = 0;
        for (int ii = 0; ii < h; ++ii) {
            for (int jj = 0; jj < w; ++jj) {
                if (buildsite[ii][jj] == '#') {
                    if (ii == 0 || jj == 0 || ii == (h-1) || jj == (w-1)) {
                        // force this region to stay grass
                        // add_edge(SOURCE, NODE(jj, ii, w), 0, 0); // leave as grass (both caps zero --> edge not necessary)
                        add_edge(NODE(jj, ii, w), SINK,   INFTY, 0); // too expensive
                    } else {
                        // add_edge(SOURCE, NODE(jj, ii, w), 0, 0); // leave as grass (both caps zero --> edge not necessary)
                        add_edge(NODE(jj, ii, w), SINK, d,   0); // digg hole
                    }
                }
                else if (buildsite[ii][jj] == '.') {
                    if (ii == 0 || jj == 0 || ii == (h-1) || jj == (w-1)) {
                        // force this region to become grass
                        // add_edge(SOURCE, NODE(jj, ii, w), f, 0); // fill hole and put grass on top 
                        cost += f;  // (edge not necessary, since flow f will always go to sink directly via next edge)
                        add_edge(NODE(jj, ii, w), SINK, INFTY, 0); // too expensive
                    } else {
                        add_edge(SOURCE, NODE(jj, ii, w), f, 0); // fill hole and put grass on top
                        // add_edge(NODE(jj, ii, w), SINK, 0, 0); // leave hole (both caps zero --> edge not necessary)
                    }
                } else {
                    assert (false);
                }

                if (jj+1 < w) {
                    add_edge(NODE(jj,   ii, w), NODE(jj+1, ii, w), b, b);
                    // add_edge(NODE(jj+1, ii, w), NODE(jj,   ii, w), b);
                }
                if (ii+1 < h) {
                    add_edge(NODE(jj, ii+1, w), NODE(jj, ii  , w), b, b);
                    // add_edge(NODE(jj, ii  , w), NODE(jj, ii+1, w), b);
                }
            }
        }

        // dump_source(w, h);
        // dump_sink(w, h);

        cost += maxflow(SOURCE, SINK);
        cout << cost << endl;

        for (int ii = 1; ii < argc; ++ii) {
            if (argv[ii][0] == 's')
                dump_solution(w, h);
            else if (argv[ii][0] == 't')
                if (is_trivial_solution(w, h))
                    cout << "trivial" << endl;
        }
    }

    return 0;
}
