// Author: Adrian Kuegel
// Date: September 12th, 2011
// Algorithm: Floyd Warshall
// Complexity: O(n^3)

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <algorithm>
#include <math.h>
using namespace std;

char name[100][101], a[101], b[101], c[101];
int x[100], y[100], sol[100],n;
bool has_shot[100], whitness[100];
double greater[100][100];

int getId(char *a) {
	for (int i=0; i<n; ++i)
		if (!strcmp(a, name[i]))
			return i;
	assert(0);
	return -1;
}

int main() {
	int tc, m;
	scanf("%d", &tc);
	while(tc--) {
		scanf("%d %d", &n, &m);
		for (int i=0; i<n; ++i) {
			scanf("%s %d %d", name[i], &x[i], &y[i]);
			has_shot[i] = false;
			whitness[i] = true;
			for (int j=0; j<n; ++j)
				greater[i][j] = -1e18;
		}
		while(m--) {
			scanf("%s heard %s firing before %s", a, b, c);
			int i = getId(a);
			int j = getId(b);
			int k = getId(c);
			double d1 = hypot(x[j]-x[i], y[j]-y[i]);
			double d2 = hypot(x[k]-x[i], y[k]-y[i]);
			// if d1 >= d2, this means that person j shot more than d1 - d2 seconds before person k
			// -> shot_time(k) - shot_time(j) > d1 - d2
			// else person j may not have fired d2 - d1 seconds or more later than person k
			// -> shot_time(j) - shot_time(k) < d2 - d1 <-> shot_time(k) - shot_time(j) > d1 - d2
			greater[k][j] = max(greater[k][j], d1 - d2);
			whitness[k] = false;
			whitness[j] = false;
		}
		for (int i=0; i<n; ++i)
			for (int j=0; j<n; ++j)
				for (int k=0; k<n; ++k)
					greater[j][k] = max(greater[j][k], greater[j][i] + greater[i][k]);
		bool impossible = false;
		for (int i=0; i<n; ++i)
			if (greater[i][i] >= 0) {
				impossible = true;
				break;
			}
		if (impossible) {
			puts("IMPOSSIBLE");
			continue;
		}
		bool unknown = false;
		int l = 0;
		while(1) {
			// check if there is someone who shoots sooner as everyone else still remaining
			int ind = -1;
			for (int j=0; j<n; ++j) {
				if (has_shot[j] || whitness[j])
					continue;
				bool poss = true;
				for (int k=0; k<n; ++k)
					if (!has_shot[k] && !whitness[k] && greater[j][k] >= 0) {
						poss = false;
						break;
					}
				if (poss) {
					if (ind >= 0)
						unknown = true;
					ind = j;
				}
			}
			if (ind < 0 || unknown)
				break;
			sol[l++] = ind;
			has_shot[ind] = true;
		}
		if (unknown)
			puts("UNKNOWN");
		else {
			for (int i=0; i<l; ++i) {
				if (i) printf(" ");
				printf("%s", name[sol[i]]);
			}
			puts("");
		}
	}
	return 0;
}
