/*
Author: Adrian Kuegel
Date: August 25th, 2011
Algorithm: Dynamic programming
Complexity: O(3^n)
*/

#include <stdio.h>
#include <string.h>
#include <algorithm>
using namespace std;

char suit[5] = "sdhc";
char value[14] = "23456789TJQKA";

int getValue(char c[3]) {
	int add;
	for (int i=0; i<4; ++i)
		if (c[1] == suit[i]) {
			add = i * 13;
			break;
		}
	for (int i=0; i<13; ++i)
		if (c[0] == value[i])
			return add + i;
	return -1;
}

bool is_straight(int *tv, int l) {
	if (l < 5)
		return false;
	for (int i=1; i<l; ++i)
		if (tv[i] != tv[i-1]+1)
			return false;
	return true;
}

bool is_fullhouse(int *tv, int l) {
	if (l != 5)
		return false;
	return (tv[0] == tv[1] && tv[2] == tv[4]) || (tv[0] == tv[2] && tv[3] == tv[4]);
}

bool isvalid[1<<13];
int dp[1<<13], used[1<<13];

void print_card(int val) {
	printf("%c%c", value[val%13], suit[val/13]);
}

bool comp(int a, int b) {
	return a % 13 < b % 13;
}

int main() {
	int tc, val[13], tv[13];
	scanf("%d", &tc);
	while(tc--) {
		char card[3];
		for (int i=0; i<13; ++i) {
			scanf("%s", card);
			val[i] = getValue(card);
		}
		sort(val, val+13, comp);
		memset(isvalid, 0, sizeof(isvalid));
		for (int i=(1<<13)-1; i>0; --i) {
			int l = 0;
			for (int j=0; j<13; ++j) {
				if (i&(1<<j))
					tv[l++] = val[j]%13;
			}
			if ((tv[l-1] == tv[0] && l <= 4) || is_straight(tv, l) || is_fullhouse(tv, l))
				isvalid[i] = true;
		}
		dp[0] = 0;
		for (int i=1; i<(1<<13); ++i) {
			dp[i] = 14;
			for (int j=i; j; j=(j-1)&i)
				if (isvalid[j] && dp[i] > dp[i^j] + 1) {
					dp[i] = dp[i^j] + 1;
					used[i] = j;
				}
		}
		int cur = (1<<13)-1;
		printf("%d\n", dp[cur]);
		while(cur) {
			bool first = true;
			for (int i=0; i<13; ++i)
				if (used[cur]&(1<<i)) {
					if (!first) printf(" ");
					else first = false;
					print_card(val[i]);
				}
			puts("");
			cur ^= used[cur];
		}
	}
	return 0;
}
