/*
Author: Adrian Kuegel
Date: August 25th
Algorithm: Quad-Tree
*/

#include <assert.h>
#include <stdio.h>
#include <math.h>
#include <vector>
#include <utility>
#include <algorithm>
using namespace std;

typedef pair<int, int> PII;

struct quadtree {
	quadtree *ch[4];
	double x1, y1, x2, y2;
	int px, py;
	quadtree(double X1, double Y1, double X2, double Y2) {
		x1 = X1;
		y1 = Y1;
		x2 = X2;
		y2 = Y2;
		px = (int)floor(x1)-1;
		for (int i=0; i<4; ++i)
			ch[i] = NULL;
	}
	~quadtree() {
		for (int i=0; i<4; ++i)
			delete ch[i];
	}
}*root;

void insert(quadtree *p, int px, int py) {
	if (p->x1 > px || p->x2 <= px || p->y1 > py || p->y2 <= py)
		return;
	// check if node already contains a point
	if (p->px >= p->x1) {
		p->ch[0] = new quadtree(p->x1, p->y1, (p->x1+p->x2)/2, (p->y1+p->y2)/2);
		p->ch[1] = new quadtree((p->x1+p->x2)/2, p->y1, p->x2, (p->y1+p->y2)/2);
		p->ch[2] = new quadtree(p->x1, (p->y1+p->y2)/2, (p->x1+p->x2)/2, p->y2);
		p->ch[3] = new quadtree((p->x1+p->x2)/2, (p->y1+p->y2)/2, p->x2, p->y2);
		for (int i=0; i<4; ++i)
			insert(p->ch[i], p->px, p->py);
		p->px = (int)floor(p->x1)-1;
	}
	// check if it is a leaf node
	if (p->ch[0] == NULL) {
		p->px = px;
		p->py = py;
		return;
	}
	for (int i=0; i<4; ++i)
		insert(p->ch[i], px, py);
}

int r, h, bx[10], by[10], ex[10], ey[10];
vector<PII> res;

#define DET(a,b,c,d)         ((a)*(d)-(b)*(c))

int ccw(const PII &a, const PII &b, const PII &c) {
	PII e(b.first-a.first, b.second-a.second);
	PII f(c.first-a.first, c.second-a.second);
	double d = DET(e.first, f.first, e.second, f.second);
	if (d >  1e-9)
		return 1;
	if (d < -1e-9)
		return -1;
	return 0;
}

bool between(const PII &a, const PII &b, const PII &c) {
	PII bb, cc;
	if (b > c) {
		bb = c;
		cc = b;
	}
	else {
		bb = b;
		cc = c;
	}
	return a >= bb && a <= cc;
}

int intersect(const PII &a, const PII &b, const PII &c, const PII &d) {
	if (a == b) {
		if (a == c || a == d)
			return true;
		if (c == d)
			return false;
		if (!ccw(c, d, a) && between(a, c, d))
			return true;
		return false;
	}
	if (!ccw(a, b, c) && between(c, a, b) || !ccw(a, b, d) && between(d, a, b))
		return true;
	if (c == d)
		return false;
	if (!ccw(c, d, a) && between(a, c, d) || !ccw(c, d, b) && between(b, c, d))
		return true;
	if (!ccw(c, d, a) && !ccw(c, d, b) && (between(a, c, d) || between(b, c, d) || between(c, a, b) && between(d, a, b)))
		return true;
	return ccw(a,b,c) * ccw(a,b,d) < 0 && ccw(c,d,a) * ccw(c,d,b) < 0;
}

void locate(quadtree *p, int x, int y) {
	if (p == NULL)
		return;
	if (p->y1 - y > r || y - p->y2 > r)
		return;
	if (x < p->x1) {
		if (p->x1 - x > r)
			return;
		if (y < p->y1 && hypot(p->y1-y, p->x1-x) > r)
			return;
		if (y > p->y2 && hypot(y-p->y2, p->x1-x) > r)
			return;
	}
	if (x > p->x2) {
		if (x - p->x2 > r)
			return;
		if (y < p->y1 && hypot(p->y1-y, p->x2-x) > r)
			return;
		if (y > p->y2 && hypot(y-p->y2, p->x2-x) > r)
			return;
	}
	// check if it is a leaf node
	if (p->px >= p->x1) {
		double dist = hypot(p->px-x, p->py-y);
		if (dist > r)
			return;
		for (int i=0; i<h; ++i)
			if (intersect(PII(bx[i], by[i]), PII(ex[i], ey[i]), PII(p->px, p->py), PII(x, y)))
				dist += 1;
		if (dist > r)
			return;
		res.push_back(PII(p->px, p->py));
		return;
	}
	for (int i=0; i<4; ++i)
		locate(p->ch[i], x, y);
}

int main() {
	int tc;
	scanf("%d", &tc);
	while(tc--) {
		int s, p, x, y;
		scanf("%d %d %d %d", &s, &r, &h, &p);
		root = new quadtree(-10000, -10000, 10000, 10000);
		for (int i=0; i<s; ++i) {
			scanf("%d %d", &x, &y);
			insert(root, x, y);
		}
		for (int i=0; i<h; ++i)
			scanf("%d %d %d %d", &bx[i], &by[i], &ex[i], &ey[i]);
		for (int i=0; i<p; ++i) {
			scanf("%d %d", &x, &y);
			res.clear();
			locate(root, x, y);
			sort(res.begin(), res.end());
			printf("%d", res.size());
			for (vector<PII>::iterator it=res.begin(); it!=res.end(); ++it)
				printf(" (%d,%d)", it->first, it->second);
			puts("");
		}
		delete root;
	}
	return 0;
}
