// @EXPECTED_RESULTS@: CORRECT

#include <stdio.h>
#include <algorithm>
#include <set>
#include <map>

using namespace std;

typedef pair<int, int> pii;
typedef pair<pii, pii> piiii;
typedef multimap<int, pii>::iterator mit;
#define FORIT(i,c) for (__typeof__((c).begin()) i = (c).begin(); i != (c).end(); i++)
#define FOR(i,a,b) for (int i = (a); i < (b); i++)
#define X(a) ((a).first)
#define Y(a) ((a).second)

#define MAX_WALLS 10
#define MAX_COORD (10000+2)

int numCircles, radius, numWalls, numQueries;
piiii wall[MAX_WALLS];
multimap<int, pii> circles;

int HASH(int a, int b) {
	a += MAX_COORD;
	b += MAX_COORD;
	int xquadr = a / radius;
	int yquadr = b / radius;
	int ret = xquadr * MAX_COORD + yquadr;
	return ret;
}

bool sqdistLE(pii a, pii b, int radius) {
	int xD = X(a) - X(b);
	int yD = Y(a) - Y(b);
	return (xD*xD + yD*yD) <= radius * radius;
}

int ccw(pii p0, pii p1, pii p2) {
	int c = (X(p1) - X(p0)) * (Y(p2) - Y(p0))
	      - (X(p2) - X(p0)) * (Y(p1) - Y(p0));
	if (c == 0) return 0;
	return (c > 0) ? 1 : -1;
}

bool onSegment(pii p0, pii p1, pii p2) {
	if (ccw(p0, p1, p2) != 0) return false;
	return X(p0) >= min(X(p1), X(p2)) && X(p0) <= max(X(p1), X(p2)) &&
	       Y(p0) >= min(Y(p1), Y(p2)) && Y(p0) <= max(Y(p1), Y(p2));
}

bool intersects(piiii a, piiii b) {
	if (ccw(X(a), Y(a), X(b)) * ccw(X(a), Y(a), Y(b)) < 0 &&
	    ccw(X(b), Y(b), X(a)) * ccw(X(b), Y(b), Y(a)) < 0) {
		return true;
	}
	if (onSegment(X(b),X(a),Y(a)) || onSegment(Y(b),X(a),Y(a))) {
		return true;
	}
	return false;
}

int main(int argc, char** argv) {
	int numCases;
	scanf("%d", &numCases);

	while (numCases --> 0) {
		circles.clear();
		scanf("%d%d%d%d", &numCircles, &radius, &numWalls, &numQueries);

		FOR(i, 0, numCircles) {
			pii circle;
			scanf("%d%d", &X(circle), &Y(circle));
			circles.insert(make_pair(HASH(X(circle), Y(circle)), circle));
		}
		FOR(i, 0, numWalls) scanf("%d%d%d%d", &X(X(wall[i])), &Y(X(wall[i])), &X(Y(wall[i])), &Y(Y(wall[i])));

		FOR(i, 0, numQueries) {
			pii interesting;
			scanf("%d%d", &interesting.first, &interesting.second);

			set<pii> result;
			for (int x = -radius; x <= radius; x += radius) {
				for (int y = -radius; y <= radius; y += radius) {
					pair<mit, mit> hit = circles.equal_range(HASH(X(interesting) + x, Y(interesting) + y));
					while (hit.first != hit.second) {
						pii circle = (hit.first)->second;
						if (sqdistLE(circle, interesting, radius)) {
							piiii connection;
							X(connection) = interesting;
							Y(connection) = circle;
							int intersect = 0;
							FOR(k, 0, numWalls) {
								if (intersects(connection, wall[k])) {
									intersect++;
								}
							}
							int newRadius = radius - intersect;
							if (newRadius > 0 && sqdistLE(circle, interesting, newRadius)) {
								result.insert(circle);
							}
						}
						hit.first++;
					}
				}
			}

			printf("%d", (int) result.size());
			FORIT(j, result) {
				printf(" (%d,%d)", j->first, j->second);
			}
			puts("");
		}
	}

	return 0;
}
