#include <stdio.h>
#include <string.h>
#include <queue>

using namespace std;

#define NMAX 111
#define INF 1000000000

int cap[NMAX][NMAX], flow[NMAX][NMAX], p[NMAX], ptype[NMAX], flowinc[NMAX];
int N, M, i, j, k, maxflow, answer;

int computeFlow(int s, int t) {
	for (int i = 0; i < N; i++)
		p[i] = ptype[i] = flowinc[i] = 0;
		
	queue<int> q;
	q.push(s);
	flowinc[s] = INF;
	while (!q.empty() > 0) {
		int i = q.front();
		q.pop();
		for (int j = 0; j < N; j++)
			if (!flowinc[j]) {
				if (flow[i][j] < cap[i][j]) {
					flowinc[j] = min(flowinc[i], cap[i][j] - flow[i][j]);
					q.push(j);
					p[j] = i;
					ptype[j] = 1;
				} else if (flow[j][i] > 0) {
					flowinc[j] = min(flowinc[i], flow[j][i]);
					q.push(j);
					p[j] = i;
					ptype[j] = 2;
				}
			}
	}
	
	return (flowinc[t] > 0);
}
	
void increaseFlow(int s, int t) {
	int x = t;
	while (x != s) {
		if (ptype[x] == 1) {
			flow[p[x]][x] += flowinc[t];
		} else {
			flow[x][p[x]] -= flowinc[t];
		}
		x = p[x];
	}
	maxflow += flowinc[t];
}
	
// cap and N must be set properly.
int computeMaxFlow(int s, int t) {	
	maxflow = 0;
	memset(flow, 0, sizeof(flow));
	while (computeFlow(s, t) != 0)
		increaseFlow(s, t);
	return maxflow;
}

int main() {
	while (scanf("%d %d", &N, &M) == 2) {
		fprintf(stderr, "N=%d M=%d\n", N, M);
		
		memset(cap, 0, sizeof(cap));
		for (k = 1; k <= M; k++) {
			scanf(" (%d,%d)", &i, &j);
			cap[i][j]++;
			cap[j][i]++;
		}
		
		answer = M;
		if (N <= 1)
			answer = 0;

		for (int i = 1; i < N; i++) {
			int flow = computeMaxFlow(0, i);
			answer = min(answer, flow);
		}
		
		printf("%d\n", answer);
	}

	return 0;
}
